import { ComponentInstanceState, CapturedArguments } from '@glimmer/interfaces';
import { PathReference, CachedReference } from './reference';
export declare const UPDATE_REFERENCED_VALUE: unique symbol;
/**
 * This module contains the references relevant to the Glimmer templating layer.
 * Fundamentally, templates consist of 3 basic types of references:
 *
 * 1. RootReference
 * 2. PropertyReference
 * 3. IterationItemReference
 *
 * Every value in a template (e.g. `{{this.some.prop}}`, `<SomeComponent/>`,
 * `{{my-helper}}`, etc) is ultimately represented by these references.
 * Components, helpers, and modifiers are represented by a single root
 * reference. Paths are represented by a chain of references, and iterables
 * consist of an array of many item references.
 */
export interface TemplatePathReference<T = unknown> extends PathReference<T> {
    [UPDATE_REFERENCED_VALUE]?: (value: T) => void;
}
/**
 * RootReferences refer to a constant root value within a template. For
 * instance, the `this` in `{{this.some.prop}}`. This is typically a:
 *
 * - Component
 * - Controller
 * - Helper
 *
 * Or another "top level" template construct, if you will. PropertyReferences
 * chain off a root reference in the template, and can then be passed around and
 * used at will.
 */
export declare abstract class RootReference<T = unknown> extends CachedReference<T> implements TemplatePathReference<T> {
    private children;
    debugLabel?: string;
    get(key: string): TemplatePathReference;
}
export declare class ComponentRootReference<T extends ComponentInstanceState> extends RootReference<T> {
    private inner;
    constructor(inner: T);
    value(): T;
    isConst(): boolean;
    compute(): T;
}
export declare type InternalHelperFunction<T = unknown> = (args: CapturedArguments) => T;
export declare class HelperRootReference<T = unknown> extends RootReference<T> {
    compute: () => T;
    constructor(fn: InternalHelperFunction<T>, args: CapturedArguments, debugName?: string);
}
/**
 * PropertyReferences represent a property that has been accessed on a root, or
 * another property (or iterable, see below). `some` and `prop` in
 * `{{this.some.prop}}` are each property references, `some` being a property of
 * `this`, and `prop` being a property of `some`. They are constructed by
 * recursively calling `get` on the previous reference as a template chain is
 * followed.
 */
export declare class PropertyReference extends CachedReference implements TemplatePathReference {
    protected parentReference: TemplatePathReference;
    protected propertyKey: string;
    private children;
    debugLabel?: string;
    constructor(parentReference: TemplatePathReference, propertyKey: string);
    compute(): unknown;
    get(key: string): TemplatePathReference;
    [UPDATE_REFERENCED_VALUE](value: unknown): void;
}
/**
 * IterationItemReferences represent an individual item in an iterable `each`.
 * They are similar to PropertyReferences, but since iteration items need to be
 * updated they have slightly different behavior. Concretely, they are the
 * `item` in:
 *
 * ```hbs
 * {{#each this.items as |item|}}
 *   {{item.foo}}
 * {{/each}}
 * ```
 *
 * Properties can chain off an iteration item, just like with the other template
 * reference types.
 */
export declare class IterationItemReference<T = unknown> implements TemplatePathReference<T> {
    parentReference: TemplatePathReference;
    private itemValue;
    private children;
    private tag;
    debugLabel?: string;
    constructor(parentReference: TemplatePathReference, itemValue: T, itemKey: unknown);
    isConst(): boolean;
    value(): T;
    update(value: T): void;
    get(key: string): TemplatePathReference;
}
//# sourceMappingURL=template.d.ts.map